"""Functions and utilities to produce features"""

import geopandas as gpd
import numpy as np
from rasterio.crs import CRS
from satio.collections import FolderCollection
from shapely.geometry import Point
from skimage.transform import resize


def get_dem(tile, bounds, epsg):
    class DEMCollection(FolderCollection):
        def _filename(self, tile, *args, **kwargs):
            utm = tile[:2]
            return self.folder / utm / f"dem_{tile}.tif"

    dem_path = "/vitodata/vegteam/auxdata/dem/s2grid/20m/"
    dem_coll = DEMCollection(dem_path)
    dem_alt = dem_coll.filter_tile(tile).filter_bounds(bounds, epsg).load()

    return dem_alt


def get_latlon(bounds, epsg, resolution=10, steps=5):
    """
    Returns a lat, lon feature from the given bounds/epsg.

    This provide a coarse (but relatively fast) approximation to generate
    lat lon layers for each pixel.

    'steps' specifies how many points per axis should be use to perform
    the mesh approximation of the canvas
    """

    xmin, ymin, xmax, ymax = bounds
    out_shape = (
        int(np.floor((ymax - ymin) / resolution)),
        int(np.floor((xmax - xmin) / resolution)),
    )

    xx = np.linspace(xmin + resolution / 2, xmax - resolution / 2, steps)
    yy = np.linspace(ymax - resolution / 2, ymin + resolution / 2, steps)

    xx = np.broadcast_to(xx, [steps, steps]).reshape(-1)
    yy = np.broadcast_to(yy, [steps, steps]).T.reshape(-1)

    points = [Point(x0, y0) for x0, y0 in zip(xx, yy)]

    gs = gpd.GeoSeries(points, crs=CRS.from_epsg(epsg))
    gs = gs.to_crs(epsg=4326)

    lon_mesh = gs.apply(lambda p: p.x).values.reshape((steps, steps))
    lat_mesh = gs.apply(lambda p: p.y).values.reshape((steps, steps))

    lon = resize(lon_mesh, out_shape, order=1, mode="edge")
    lat = resize(lat_mesh, out_shape, order=1, mode="edge")

    return lat, lon
